/*
 *	$Id: simple_client.c,v 1.2 2005/05/02 23:44:21 rh Exp $
 *
 * a simple TCP client that uses a separate input and output task
 *
 */

#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <netdb.h>
#include <errno.h>
#include <sys/wait.h>

#include "tcpsocket.h"

/* gucis has no hstrerror() -- you can comment this out on other systems */
#define hstrerror(e) ((e) == HOST_NOT_FOUND ? "host not found" :	\
		     ((e) == NO_ADDRESS || (e) == NO_DATA ? "not a host" :\
		     ((e) == NO_RECOVERY ? "non-recoverable error" :	\
		     ((e) == TRY_AGAIN ? "try again later" :		\
		     ((e) ? "unknown error" : "name resolved ok") ))))

int handle_input(TCPSocket *sock)
{
	FILE *in = TCPSocket_getInputStream(sock);
	char buffer[82];

	if (in == NULL)
	{
		perror("Cannot get input stream");
		return EXIT_FAILURE;
	}

	if (fgets(buffer, sizeof(buffer), in) == NULL)
	{
		perror("Cannot read data from network");
		return EXIT_FAILURE;
	}

	printf("child: server said result was %s", buffer);

	return EXIT_SUCCESS;
}


int handle_output(TCPSocket *sock)
{
	FILE *out = TCPSocket_getOutputStream(sock);
	char *data = "4 5 + 2 6 - / p\n";

	printf("parent: sending the following data to the server: %s", data);

	if (fputs(data, out) < 0 || fflush(out) < 0)
	{
		perror("cannot write to the stream");
		return EXIT_FAILURE;
	}

	return EXIT_SUCCESS;
}


int main (int argc, char *argv[])
{
	TCPSocket *sock;
	char *host = "localhost";
	int port = 12345;
	int rv, pid, status;

	if (argc > 1) host = argv[1];		/* get destination host */
	if (argc > 2) port = atoi(argv[2]);	/* get destination port */

	sock = new_TCPSocket(host, port);	/* establish connection */

	if (sock == NULL)			/* can't connect? */
	{
		fprintf(stderr, "Can't connect to %s at port %d: %s (%s)\n",
			host, port, errno ? strerror(errno) : "",
			hstrerror(h_errno));
		return EXIT_FAILURE;
	}

	/*
	 * create a new process:
	 * 	- parent handles output
	 *	- child handles input
	 */
	switch (pid = fork())			/* create child process */
	{
		case -1:
			perror("cannot fork");
		break;

		case 0:				/* child */
			rv = handle_input(sock);
		break;

		default:			/* parent */
			rv = handle_output(sock);
			wait(&status);		/* wait for child */
			printf("parent: child returned %d\n", status);
	}

	TCPSocket_free(sock);

	return rv;
}
